@@ -4,6 +4,7 @@ import struct
 import array
 
 from . import units
+from .compat import int_from_byte
 
 class Dictionary(object):
     """
@@ -12,9 +13,8 @@ class Dictionary(object):
     def __init__(self):
         self._units = array.array(str("I"))
 
-    def root(self):
-        "Root index"
-        return 0
+    ROOT = 0
+    "Root index"
 
     def has_value(self, index):
         "Checks if a given index is related to the end of a key."
@@ -33,14 +33,14 @@ class Dictionary(object):
 
     def contains(self, key):
         "Exact matching."
-        index = self.follow_bytes(key, self.root())
+        index = self.follow_bytes(key, self.ROOT)
         if index is None:
             return False
         return self.has_value(index)
 
     def find(self, key):
         "Exact matching (returns value)"
-        index = self.follow_bytes(key, self.root())
+        index = self.follow_bytes(key, self.ROOT)
         if index is None:
             return -1
         if not self.has_value(index):
@@ -59,8 +59,8 @@ class Dictionary(object):
 
     def follow_bytes(self, s, index):
         "Follows transitions."
-        for ch in bytearray(s):
-            index = self.follow_char(ch, index)
+        for ch in s:
+            index = self.follow_char(int_from_byte(ch), index)
             if index is None:
                 return None
 
@@ -76,11 +76,10 @@ class Dictionary(object):
 
 class Guide(object):
 
-    def __init__(self):
-        self._units = None
+    ROOT = 0
 
-    def root(self):
-        return 0
+    def __init__(self):
+        self._units = array.array(str("B"))
 
     def child(self, index):
         return self._units[index*2]
@@ -90,10 +89,10 @@ class Guide(object):
 
     def read(self, fp):
         base_size = struct.unpack(str("=I"), fp.read(4))[0]
-        self._units = bytearray(fp.read(base_size*2))
+        self._units.fromfile(fp, base_size*2)
 
     def size(self):
-        return len(self._units) if self._units is not None else 0
+        return len(self._units)
 
 
 class Completer(object):
@@ -110,7 +109,7 @@ class Completer(object):
 
         if self._guide.size():
             self._index_stack = [index]
-            self._last_index = self._dic.root()
+            self._last_index = self._dic.ROOT
         else:
             self._index_stack = []
 
@@ -123,7 +122,7 @@ class Completer(object):
 
         index = self._index_stack[-1]
 
-        if self._last_index != self._dic.root():
+        if self._last_index != self._dic.ROOT:
 
             child_label = self._guide.child(index) # UCharType
 
